#ifndef __RT_RENDERER__
#define __RT_RENDERER__

#include <math.h>

class MyLightSource : public sgRayTracingLightSource::sgCPointLightSource
{
public:
  virtual  SG_POINT                       GetLocation() const
  {
    SG_POINT resP = {150.0, -150.0, 150.0};
    return resP;
  }
  virtual  sgRayTracingTexture::RT_COLOR  GetColor()   const
  {
    sgRayTracingTexture::RT_COLOR col = {1.0, 1.0, 1.0, 0.0};
    return col;
  }
  virtual  bool                           GetAtmosphericInteraction() const
  {return false;};
};

class MyRenderer : public SG_VIEW_PORT,
	public sgCMaterialsLibrary,
	public sgCLightSourcesContainer
{
  HBITMAP    m_bitmap;
  HBITMAP    m_oldBitmap;
  HDC        m_Compat_hdc;
  HDC        m_hDC;
  SIZE       m_draw_sizes;
  SIZE       m_shifts;

  MyLightSource m_lS;
  
public:
  virtual sgCMaterial*  GetMaterial(const sgCObject* ooo);

  virtual unsigned int    GetLightSourcesCount()
  {
	  return 1;
  }
  virtual const sgRayTracingLightSource::sgCLightSource*  GetLightSource(unsigned int ind)
  {
	   return &m_lS;
  }


  virtual int       GetWidth();
  virtual int       GetHeight();
  virtual void      EndRender() {};

  virtual void __cdecl  RenderPixel(int x, int y,
    double pixRed, double pixGreen, double pixBlue, double pixAlpha)
  {
    if (!m_bitmap || !m_Compat_hdc)
      return;

    ::SetPixel(m_Compat_hdc,x,y,RGB((int)floor(pixRed*255.0),
                    (int)floor(pixGreen*255.0),
                    (int)floor(pixBlue*255.0)));

    if ((!(y%10) && x==0) || (y==m_draw_sizes.cy-1))
    {
      BitBlt(m_hDC, m_shifts.cx, m_shifts.cy, m_draw_sizes.cx, m_draw_sizes.cy,
        m_Compat_hdc, 0, 0, SRCCOPY);
    }

  };

  void InitRender(HDC  hDC, unsigned int width, unsigned int height,
                          unsigned int shift_x,  unsigned int shift_y)
  {
    DeInitRender();

    m_Compat_hdc = CreateCompatibleDC(hDC);
    m_hDC  = hDC;

    BITMAPINFO bmi;
    VOID *pvbits;

    ZeroMemory(&bmi.bmiHeader,sizeof(bmi.bmiHeader));
    bmi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
    bmi.bmiHeader.biHeight = height;
    bmi.bmiHeader.biWidth = width;
    bmi.bmiHeader.biPlanes = 1;
    bmi.bmiHeader.biBitCount = 24;
    bmi.bmiHeader.biCompression = BI_RGB;
    bmi.bmiHeader.biSizeImage = -1 * bmi.bmiHeader.biHeight * bmi.bmiHeader.biWidth * 4;

    m_bitmap = CreateDIBSection(NULL,&bmi,DIB_RGB_COLORS,&pvbits,NULL,0x0);

    m_oldBitmap = (HBITMAP)SelectObject(m_Compat_hdc, (HBITMAP)m_bitmap);

    m_draw_sizes.cx = width;
    m_draw_sizes.cy = height;
    m_shifts.cx = shift_x;
    m_shifts.cy = shift_y;
  };

  void  DrawOnDC(HDC distDC)
  {
    if (distDC && m_Compat_hdc)
      BitBlt(distDC, m_shifts.cx, m_shifts.cy, m_draw_sizes.cx, m_draw_sizes.cy,
      m_Compat_hdc, 0, 0, SRCCOPY);
  }

  void  DeInitRender()
  {
    if (m_oldBitmap && m_Compat_hdc)
    {
      SelectObject(m_Compat_hdc, (HBITMAP)m_oldBitmap);
      if (m_bitmap)
        DeleteObject(m_bitmap);
      DeleteDC(m_Compat_hdc);
    }
    m_bitmap = NULL;
    m_oldBitmap = NULL;
    m_Compat_hdc = NULL;
    m_hDC  = NULL;
    m_draw_sizes.cx = m_draw_sizes.cy = 0;
    m_shifts.cx = m_shifts.cy = 0;
  }

  MyRenderer()
  {
    m_bitmap = NULL;
    m_oldBitmap = NULL;
    m_Compat_hdc = NULL;
    m_hDC  = NULL;
    m_draw_sizes.cx = m_draw_sizes.cy = 0;
    m_shifts.cx = m_shifts.cy = 0;
  };

  ~MyRenderer()
  {
    DeInitRender();
  }

};

extern MyRenderer global_rend;
#endif